import java.awt.*;
import java.util.Vector;

public class Arrangement {

  final static int STRING = 0;
  final static int VBOX = 1;   // vertical arrangement, with rectangle
  final static int HBOX = 2;   // horizontal arrangement, with rectangle
  final static int VPLAIN = 3; // vertical arrangement
  final static int HPLAIN = 4; // horizontal arrangement
  final static int VTREE = 5;  // 
  final static int HTREE = 6;  // 

  final static int ALL_CLAUSES  = 1;
  final static int ALL_IDS      = 2;
  final static int MAIN_CLAUSES = 3;

  Color bg = null;          // null background means "transparent"
  Color fg = null;          // null foreground means "default"
  int pad = 5;
  boolean stagger = true;

  // Variables changed at different times

  boolean full_clause = true;  
  boolean abbreviated = false;
  Color highlight = null;       // if not null, highlight

  // Filled in during construction

  Arrangement parent;
  Arrangement next;
  Arrangement prev;
  Arrangement first_child;
  Arrangement last_child;

  int type;                // see "constant variables" above
  Proof_node proof_node;    // proof node (if any) this arrangement represents
  Vector emphasize_vector;  // position (if any) that should be emphasized
  Color emphasize_color;    // for emphasized text

  // Filled in by relative_arrangement:

  String label;
  int xo, yo;                 // offsets from parent
  int hi_x, hi_y;             // rectangle to highlight when selected
  int hi_width, hi_height;    //     (polygon may be highlishted instead)
  int width, height;          // size of the arrangement

  int lab_width, lab_height;  // label size and poisition
  int lab_xo, lab_yo;
  int lab_center_x;
  int lab_center_y;
  int link_up_x, link_up_y;
  int link_dn_x, link_dn_y;

  boolean emphasize;               // should part of the label be amphasized?
  int lab_em_start, lab_em_width;  // positions of emphazized substring of label

  X_polygon poly;

  // Filled in by absolute_arrangement:

  int x, y;           // place to draw it (absolute)


  // The main constructor method

  public Arrangement(Proof_node pn, Vector already_arranged, int depth) {
    fill_in_arrangement(pn, already_arranged, depth);
  }  // Arrangement constructor

  public void fill_in_arrangement(Proof_node pn, Vector already_arranged,
				  int depth) {
    proof_node = pn;
    type = VTREE;

    if (already_arranged.contains(pn) || depth == 0) {
      if (pn.parents.length > 0)
	abbreviated = true;
    }
    else {
      // Get subproof arrangements, if any.
      for (int i = 0; i < pn.parents.length; i++) {
	Arrangement a1 = new Arrangement(pn.parents[i], already_arranged,
					 depth-1);
	a1.parent = this;
	a1.prev = last_child;
	if (last_child != null)
	  last_child.next = a1;
	else
	  first_child = a1;
	last_child = a1;
	a1.emphasize_vector = pn.positions[i];
	switch (pn.type) {
	case pn.RESOLVE: a1.emphasize_color = Color.yellow; break;
	case pn.MERGE:   a1.emphasize_color = Color.magenta; break;
	case pn.PARAMOD: a1.emphasize_color = Color.pink; break;
	case pn.FLIP:    a1.emphasize_color = Color.orange; break;
	default:         a1.emphasize_color = Color.lightGray; break;
	}
	// Also emphasize positions before instantiation.
	if (pn.type == pn.RESOLVE || pn.type == pn.PARAMOD ||
	    pn.type == pn.MERGE) {
	  if (a1.first_child != null && 
	      a1.proof_node.type == pn.INSTANTIATE) {
	    a1.first_child.emphasize_vector = a1.emphasize_vector;
	    a1.first_child.emphasize_color = a1.emphasize_color;
	  }
	}
      }
      already_arranged.addElement(pn);
    }
  }  // fill_in_arrangement

  public void abbreviate(FontMetrics fm, boolean transpose_label) {
    // This is a little dirty, because I don't want to rearrange everything.
    if (first_child != null) {
      first_child = null;
      last_child = null;
      abbreviated = true;
      int new_x_offset = xo + lab_xo;	
      int new_y_offset = yo + lab_yo;	
      relative_arrangement(fm, transpose_label);
      if (transpose_label)
	transpose();
      xo = new_x_offset;
      yo = new_y_offset;
      if (parent != null)
        parent.absolute(parent.x, parent.y);
      else
        absolute(x,y);
    }
  }  // abbreviate

  public void unabbreviate(int level) {
    if (!abbreviated) {
      first_child = null;
      last_child = null;
    }
    // Construct a whole new arrangement for this node.
    abbreviated = false;
    fill_in_arrangement(proof_node, new Vector(), level);
  }  // unabbreviate

  public void relative_arrangement(FontMetrics fm, boolean transpose_label) {
    // This routine does all the geometry calculations.
    for (Arrangement a = first_child; a != null; a = a.next)
      a.relative_arrangement(fm, transpose_label);
    relative_arrange_this_node(fm, transpose_label);
  }  // relative_arrangement

  public void relative_arrange_this_node(FontMetrics fm,
					 boolean transpose_label) {
    
    // Label size and emphasis.

    String id;
    if (proof_node.id2 == null)
      id = proof_node.id;
    else
      id = proof_node.id + " [" + proof_node.id2 + "]";

    if (!full_clause) {
      label = id;
      emphasize = false;
    }
    else {
      Hi_string hs;
      hs = new Hi_string(Hi_string.CLAUSE, id + ": ",
			 proof_node.clause, emphasize_vector, null);
      emphasize = (emphasize_vector != null);
      label = hs.s;
      if (emphasize) {
	lab_em_start = fm.stringWidth(label.substring(0, hs.start));
	lab_em_width = fm.stringWidth(label.substring(hs.start, hs.stop));
      }
    }

    lab_width = fm.stringWidth(label);
    lab_height = fm.getAscent() + fm.getDescent();
    if (transpose_label) {
      int tmp = lab_width; lab_width = lab_height; lab_height = tmp;
      }	

    // Vertical dimensions.

    // Find height of tallest child.

    int max_height = 0;
    for (Arrangement a = first_child; a != null; a = a.next)
      max_height = Math.max(max_height, a.height);

    if (stagger) {
      // I need a simpler algorithm for this.
      boolean first_max_height_processed = false;
      int extra_height = 0;
      for (Arrangement a = first_child; a != null; a = a.next) {
	if (a.height == max_height && !first_max_height_processed) {
	  first_max_height_processed = true;
	  a.yo = 0;
	}
	else if (a.height == max_height) {
	  int r = (int) (lab_height * Math.random());
	  a.yo = (max_height - a.height) + r;
	  extra_height = Math.max(extra_height, r);
	}
	else {
	  int r = (int) (lab_height * Math.random());
	  a.yo = (max_height - a.height) - Math.min(r, max_height-a.height);
	}
      }
      max_height += extra_height;
    }
    else {
      for (Arrangement a = first_child; a != null; a = a.next)
	a.yo = (max_height - a.height);
    }

    if (first_child == null)
      height = lab_height;
    else {
      int space;
      if (transpose_label)
	space = (full_clause ? 2*lab_width : lab_width);
      else
	space = (full_clause ? 2*lab_height : lab_height);
      height = max_height + space + lab_height;
    }

    // Horizontal dimensions.

    int xoff = 0;
    int min = 0;  // Check for negative x offsets.
    for (Arrangement a = first_child; a != null; a = a.next) {
      a.xo = xoff;
      if (a.next != null) {
	int dist = squeeze(a, a.next);
	if (transpose_label)
	  dist += lab_width;
	// Dist is the offset difference of the two arrangements.
	// If negative, a.next starts to left of a!
	xoff += (dist + pad);
	min = Math.min(min, xoff);
      }
      else
	xoff += a.width;
    }

    // If any child has negative offset, adjust so that none is negative.

    if (min < 0) {
      int increment = -min;
      xoff += increment;
      for (Arrangement a = first_child; a != null; a = a.next) {
	a.xo += increment;
      }
    }

    // Find total width of arranged children (whose rectangles can overlap).

    width = 0;
    for (Arrangement a = first_child; a != null; a = a.next)
      width = Math.max(width, a.xo + a.width);

    // If label is wider than kids, center kids and reset width.

    if (lab_width > width) {
      // Move children to the right so they are centered with the label.
      for (Arrangement a = first_child; a != null; a = a.next)
	a.xo += (lab_width - width)/2;
      width = lab_width;
    }

    // Decide where to put the label.

    lab_yo = height - lab_height;
    if (first_child == null)
      lab_center_x = width/2;
    else {
      // Center between first and last child labels.
      int x1 = first_child.xo + first_child.lab_center_x;
      int x2 = last_child.xo + last_child.lab_center_x;
      lab_center_x = (x1 + x2)/2;
    }
    lab_xo = lab_center_x - lab_width/2;

    // Make sure the label is in the arangement.

    if (lab_xo < 0) lab_xo = 0;
    if (lab_xo+lab_width > width) lab_xo = width - lab_width;

    // Make the label highlight area a little bigger than the label.

    hi_x = lab_xo - pad;
    hi_y = lab_yo - pad;
    hi_width = lab_width + 2*pad;
    hi_height = lab_height + 2*pad;

    // Now, get the convex polygon for the whole arrangement.

    setup_polygon();

    // Finally, set some dependent variables;

    lab_center_y = yo + (lab_height/2);
    link_up_x = lab_center_x;
    link_up_y = lab_yo;
    link_dn_x = lab_center_x;
    link_dn_y = lab_yo + lab_height;
  }  // relative_arrange_this_node

  private void setup_polygon() {	

    // First, get the polygon for the label.  (It is a rectangle.)

    if (first_child == null) {
      // It needs some height, so it can be selected.
      int[] xp = {lab_xo, lab_xo, lab_xo + lab_width, lab_xo + lab_width};
      int[] yp = {lab_yo + lab_height, lab_yo, lab_yo, lab_yo + lab_height};
      poly = new X_polygon(xp, yp, 4);
    }
    else {
      // It needs no height, because it will be extended with child polys.
      int[] xp = {lab_xo, lab_xo + lab_width};
      int[] yp = {lab_yo + lab_height, lab_yo + lab_height};
      poly = new X_polygon(xp, yp, 2);
    }

    // Now, add each child, extending the poly. and making it convex.

    for (Arrangement a = first_child; a != null; a = a.next) {
      poly = poly.extend(a.poly, a.xo, a.yo);
    }
  }  // setup_polygon

  // The absolute() method sets the drawing position
  // for each node in the arrangement.  (The arrangement is
  // constructed bottom-up with offests, then this method sets
  // the positions from the top down.)

  public void absolute(int xpos, int ypos) {
    x = xpos;
    y = ypos;

    for (Arrangement a = first_child; a != null; a = a.next) {
      a.absolute(xpos + a.xo, ypos + a.yo);
    }
  } // absolute

  // This method determines whether a point is in an arrangement.
  // Return the smallest arrangement that contains the point.
  // For trees, use the polygon.

  public Arrangement find(int xt, int yt) {
    boolean inside;
    if (type == VTREE || type == HTREE)
      inside = poly.my_inside(xt - x, yt - y);
    else
      inside = (xt>=x && xt<=(x+width) && yt>=y && yt<=(y+height));

    if (inside) {
      // It is in this arrangement.  Find out if it is in a sub-arrangement.
      for (Arrangement a = first_child; a != null; a = a.next) {
	Arrangement aa = a.find(xt, yt);
	if (aa != null)
	  return aa;
      }
      // Not in a sub-box, so return this box.
      return this;
    }
    else
      // Not in this box.
      return null;
  }  // find

  public Arrangement find_sub_arr(Proof_node pn) {
    if (proof_node == pn)
      return(this);
    else {
      for (Arrangement a = first_child; a != null; a = a.next) {
	Arrangement af = a.find_sub_arr(pn);
	if (af != null)
	  return(af);
      }
      return null;
    }
  }  // find_sub_arr

  public Vector position() {
    if (parent == null)
      return new Vector();
    else {
      int i = 0;
      for (Arrangement a = parent.first_child; a != this; a = a.next)
	i++;
      Vector v = parent.position();
      v.addElement(new Integer(i));
      return v;
    }
  }  // position

  public Arrangement at_position(Vector v) {
    if (v.size() == 0)
      return this;
    else {
      int n = ((Integer) v.firstElement()).intValue();
      int i = 0;
      Arrangement a = first_child;
      while (i < n) {
	a = a.next;
	i++;
      }
      v.removeElementAt(0);
      return a.at_position(v);
    }
  }  // at_position

  private static int squeeze(Arrangement a1, Arrangement a2) {
    int max = a1.width;                        // max separation
    int min = a1.lab_center_x-a2.lab_center_x; // min separation (may be neg.)
    int cur = a1.width;                        // current separation
    boolean overlap = false;
    int interval = max - min;

    int yoff = a2.yo - a1.yo;  // y offset does not change;

    // System.out.println("\nsqueezing: " + a1.label + ",   " + a2.label);
    // System.out.println(a1.poly);
    // System.out.println(a2.poly);

    do {
      // System.out.println("overlap: " + overlap + ", interval=" + interval +
      // 		 ", cur=" + cur);
      interval = interval / 2;
      cur = (overlap ? cur+interval : cur-interval);
      overlap = X_polygon.overlap(a1.poly, a2.poly, cur, yoff);
    } while (interval > 4);

    if (overlap) {
      // System.out.println("  overlap at end");
      cur += interval;
    }
    // System.out.println("squeeze out: interval=" + interval + ", cur=" + cur);
    return cur;
  }  // squeeze

  public void reset_clause_display(int display) {
    if (display == ALL_CLAUSES)
      full_clause = true;
    else if (display == ALL_IDS)
      full_clause = false;
    else
      full_clause = (proof_node.id2 != null);
	  
    for (Arrangement a = first_child; a != null; a = a.next) {
      a.reset_clause_display(display);
    }
  }  // reset_clause_display

  public void transpose() {
    for (Arrangement a = first_child; a != null; a = a.next)
      a.transpose();

    int tmp;
    tmp = x; x = y; y = tmp;
    tmp = width; width = height; height = tmp;
    tmp = xo; xo = yo; yo = tmp;
    tmp = lab_xo; lab_xo = lab_yo; lab_yo = tmp;
    tmp = lab_width; lab_width = lab_height; lab_height = tmp;
    tmp = hi_x; hi_x = hi_y; hi_y = tmp;
    tmp = hi_width; hi_width = hi_height; hi_height = tmp;
    tmp = lab_center_x; lab_center_x = lab_center_y; lab_center_y = tmp;
    tmp = link_up_x; link_up_x = link_up_y; link_up_y = tmp;
    tmp = link_dn_x; link_dn_x = link_dn_y; link_dn_y = tmp;

    for (int i = 0; i < poly.npoints; i++) {
      tmp = poly.xpoints[i];
      poly.xpoints[i] = poly.ypoints[i];
      poly.ypoints[i] = tmp;
    }
  }  // transpose

}  // class Arrangement
