import java.awt.*;
import java.util.Vector;

class Draw_panel extends Scrollable_canvas {

  Prooftran prooftran;
  Arrangement full_arr;       // Full Arrangement
  Arrangement selected_arr;   // Selected Arangement

  int pad = 10;	  // around the whole thing

  boolean double_buffer = false;  // ok for java, broken for netscape
  Image buffer = null;
  int buffer_width = 0, buffer_height = 0;

  public Draw_panel(Prooftran prooftran) {
    this.prooftran = prooftran;
    setBackground(Color.white);
  }

  // paint_arrangement -- recursive paint routine

  private void paint_arrangement(Graphics g, Arrangement a) {
    if (rect_invisible(a.x, a.y, a.width, a.height))
      return;  // Entire rectangle is invisible, so don't bother drawing it.
    else {
      if (a.bg != null) {
	g.setColor(a.bg);
	g.fillRect(a.x, a.y, a.width, a.height);
	g.setColor(getForeground());
      }
      if (a.highlight != null) {
	g.setColor(a.highlight);
	if (prooftran.highlight == prooftran.POLYGON) {
	  a.poly.offset(a.x, a.y);
	  g.fillPolygon(a.poly);
	  a.poly.offset(-a.x, -a.y);
	}
	else
	  g.fillRect(a.x + a.hi_x, a.y + a.hi_y, a.hi_width, a.hi_height);
	g.setColor(getForeground());
      }

      // Paint children.

      for (Arrangement cld = a.first_child; cld != null; cld = cld.next)
	paint_arrangement(g, cld);

      // Paint objects for this node.

      if (a.type == a.HBOX || a.type == a.VBOX)
	g.drawRect(a.x, a.y, a.width, a.height);

      else if (a.type == a.STRING)
	g.drawString(a.label, a.x, a.y + g.getFontMetrics().getAscent());

      else if (a.type == a.VTREE) {

	if (a.emphasize) {
	  g.setColor(a.emphasize_color);
	  g.fillRect(a.x + a.lab_xo + a.lab_em_start,
		     a.y + a.lab_yo,
		     a.lab_em_width,
		     a.lab_height);
	  g.setColor(getForeground());
	}
	g.drawString(a.label, a.x + a.lab_xo,
		     a.y + a.lab_yo + g.getFontMetrics().getAscent());

	if (a.abbreviated)
	  g.drawRect(a.x, a.y, a.width, a.height);

	if (false) {
	  int xc = a.x + a.lab_center_x;
	  int yc = a.y + a.lab_yo;
	  for (Arrangement cld = a.first_child; cld != null; cld = cld.next) {
	    int xp = a.x + cld.xo + cld.lab_center_x;
	    int yp = a.y + cld.yo + cld.height;
	    g.drawLine(xc, yc, xp, yp);
	  }
        }
	else {
	  for (Arrangement a1 = a.first_child; a1 != null; a1 = a1.next) {
	    g.drawLine(a.x + a.link_up_x, a.y + a.link_up_y,
		 a.x + a1.xo + a1.link_dn_x, a.y + a1.yo + a1.link_dn_y);
	  }
	}
      }
    }
  }  // paint_arrangement

  // paint -- main paint routine

  public void paint(Graphics g) {
    // System.out.println("paint");

    if (full_arr == null) {
      g.translate(0, 0);
      g.clearRect(0, 0, size().width, size().height);
      set_scrollable(100, 100, g);
      g.drawString("No Proof", 50, 50);
    }
    else if (double_buffer) {
      Graphics bg;
      int w = size().width;
      int h = size().height;
      if (buffer==null || buffer_width!=w || buffer_height!=h) {
	if (buffer != null)
	  buffer.getGraphics().dispose();
	buffer = createImage(w, h);
	buffer_width = w;
	buffer_height = h;
	bg = buffer.getGraphics();
      }
      else {
	bg = buffer.getGraphics();
	bg.translate(0, 0);
	bg.clearRect(0, 0, buffer_width, buffer_height);
      }

      set_scrollable(full_arr.width + 2*pad, full_arr.height + 2*pad, bg);
      paint_arrangement(bg, full_arr);
      // buffer.flush();  // no help
      g.drawImage(buffer, 0, 0, this);
      // buffer.flush();  // no help
    }
    else {
      g.translate(0, 0);
      g.clearRect(0, 0, size().width, size().height);
      set_scrollable(full_arr.width + 2*pad, full_arr.height + 2*pad, g);
      paint_arrangement(g, full_arr);
    }
  }  // paint

  public void update(Graphics g) {
    paint(g);
  }  // update

  public void my_repaint() {
    // System.out.println("my_repaint");
    repaint();
    // The following seems to partly solve two problems I don't understand:
    // 1. Sometimes too much gets painted (in Netscape).
    // 2. Sometimes too little gets painted (with jdk java interpreter).
    // This usually works, but not always.
    // show(false);
    // show(true);
  }  // my_repaint

  public boolean handleEvent(Event evt) {
    if (evt.id != Event.MOUSE_DOWN)
      return super.handleEvent(evt);  // to handle scrolling events
    else if (full_arr == null) {
      return true;
    }
    else {
      Arrangement prev = selected_arr;
      // get virtual coordinates of click
      int x = a2v_x(evt.x);
      int y = a2v_y(evt.y);
      selected_arr = full_arr.find(x,y);
      if (prev != selected_arr) {
	if (double_buffer) {
	  if (prev != null)
	    prev.highlight = null;  // transparent
	  if (selected_arr != null) {
	    selected_arr.highlight = Color.lightGray;
	  }
	  repaint();
	}
	else {
	  // No double buffer. Draw parts directly. This prevents some flicker.
	  Graphics g = getGraphics();
	  // The following just sets the translation for this Graphics.
	  set_scrollable(full_arr.width + 2*pad, full_arr.height + 2*pad, g);
	  if (prev != null) {
	    prev.highlight = getBackground();
	    paint_arrangement(g, prev);
	    prev.highlight = null;  // transparent
	  }
	  if (selected_arr != null) {
	    selected_arr.highlight = Color.lightGray;
	    paint_arrangement(g, selected_arr);
	  }
	}
      }
      return true;
    }
  }  // handleEvent

}  // class Draw_panel
