import java.io.*;
import java.util.Vector;

public class Hi_string {

  final static String[][] var_map = {{"v0", "x"},
				     {"v1", "y"},
				     {"v2", "z"},
				     {"v3", "u"},
				     {"v4", "v"},
				     {"v5", "w"},
				     {"v64", "v0"},
				     {"v65", "v1"},
				     {"v66", "v2"},
				     {"v67", "v3"},
				     {"v68", "v4"},
				     {"v69", "v5"}};
  final static int FORMULA = 1;
  final static int CLAUSE  = 2;
  final static int LITERAL = 3;
  final static int TERM    = 4;

  int start = -1;
  int stop  = -1;
  String s;

  Hi_string(String s) {
    this.s = s;
  }  // constructor (simple)

  Hi_string(int type, String id, Cons thing, Vector pos, Cons syms) {
    StringBuffer sb = new StringBuffer();
    sb.append(id);
    if (thing.end())
      sb.append("$FALSE");
    else {
      Cons target;
      // System.out.println("Hi_string: " + thing + pos);
      if (pos == null)
	target = null;
      else
	target = thing.sub_at_pos(pos);
      for (Cons c = thing; !c.end(); c = c.tail) {
	Cons lit = c.head;
	if (c != thing)
	  sb.append(" | ");
	if (lit == target) start = sb.length();
	if (!lit.neg_literal())
	    recurse(lit.atom(), target, sb);
	else {
	  // Negative equality is a special case.
	  Cons atom = lit.tail.head;
	  if (atom.length() == 3 && atom.head.label.equals("=")) {
	    // Note that if atom of neg. eq. is target, the whole literal
	    // will be be highlighted.
	    if (atom == target) start = sb.length();
	    recurse(atom.tail.head, target, sb);
	    sb.append(" != ");
	    recurse(atom.tail.tail.head, target, sb);
	    if (atom == target) stop = sb.length();
	  }
	  else {
	    sb.append("~");
	    recurse(lit.atom(), target, sb);
	  }
	}
	if (lit == target) stop = sb.length();
      }
    }
    s = sb.toString();
  // System.out.println("start=" + start + " stop=" + stop + " " + s);
  }  // constructor (full)

  private void recurse(Cons c, Cons target, StringBuffer sb) {
    // System.out.println("recurse: " + c + " " + target + " " + sb);
    if (c == target) start = sb.length();
    if (!c.list()) {
      sb.append(variable_map(c.label));
    }
    else if (c.length() == 3 && c.head.label.equals("=")) {
      recurse(c.tail.head, target, sb);
      sb.append(" = ");
      recurse(c.tail.tail.head, target, sb);
    }
    else if (c.length() == 3 && c.head.label.equals("*")) {
      sb.append("(");
      recurse(c.tail.head, target, sb);
      sb.append("*");
      recurse(c.tail.tail.head, target, sb);
      sb.append(")");
    }
    else if (c.length() == 3 && c.head.label.equals("+")) {
      sb.append("(");
      recurse(c.tail.head, target, sb);
      sb.append("+");
      recurse(c.tail.tail.head, target, sb);
      sb.append(")");
    }
    else if (c.length() == 1) {
      sb.append(c.head.label);
    }
    else {
      sb.append(c.head.label + "(");
      for (Cons t = c.tail; !t.end(); t = t.tail) {
	if (t != c.tail) sb.append(",");
	recurse(t.head, target, sb);
      }
      sb.append(")");
    }
    if (c == target) stop = sb.length();
  }  // recurse

  private static String variable_map(String s) {
    for (int i = 0; i < var_map.length; i++) {
      if (s.equals(var_map[i][0]))
	return var_map[i][1];
    }
    return s;
  }  // variable_map

  public static void main(String args[]) throws NumberFormatException,
      IOException, Cons_exception {
    if (args.length != 1) {
      System.out.println("need one argument");
      System.exit(-1);
    }
    Cons c = Cons.get_list_from_file(args[0]);
    System.out.println("term:    " + c.head);
    System.out.println("vec:     " + c.tail.head);
    Vector v = c.tail.head.to_vector();
    Cons sub = c.head.sub_at_pos(v);
    System.out.println("subterm: " + sub);

    Hi_string hs = new Hi_string(CLAUSE, "99. ", c.head, v, null);
    System.out.println(hs.s + "start=" + hs.start + " stop=" + hs.stop + " ");

    int i;
    for (i = 0; i < hs.start; i++) System.out.print(" ");
    for (; i < hs.stop; i++) System.out.print("^");
    System.out.println("");
  }  // main
}  // class Hi_string
