import java.io.*;
import java.util.Vector;
import java.util.Hashtable;

public class Proof_object {

  // Note that this is an upside-down tree: one child and many parents.
  // Actually, it is a graph, because parents can be shared.

  public static final int OTTER_1 = 1;
  public static final int OTTER_2 = 2;
  public int type = OTTER_1;

  Proof_node proof;
  Hashtable hashtab;

  public Proof_object(Cons c) throws NumberFormatException, Proof_exception {

    // Hashtable: key=step-ID, object=Proof_node.

    hashtab = new Hashtable(2 * c.length());
    Proof_node pn = null;
    for (Cons c1 = c; !c1.end(); c1 = c1.tail) {
      Cons step = c1.head;
      pn = new Proof_node(step, hashtab);
      // If any node has an id2, the proof object is type OTTER_2;
      if (pn.id2 != null)
	type = OTTER_2;
      hashtab.put(step.head.toString(), pn);
    }
    proof = pn;  // last step becomes root of proof
  }  // Proof_object constructor 

  private void st_rec (StringBuffer sb, Proof_node pn, int depth) {
    if (pn == null)
      return;
    else {
      for (int i = 0; i < depth; i++)
	sb.append("-");
      sb.append(pn.toString());
      for (int i = 0; i < pn.parents.length; i++)
	st_rec(sb, pn.parents[i], depth+1);
    }
  }  // st_rec

  public String toString () {
    StringBuffer sb = new StringBuffer();
    st_rec(sb, proof, 0);
    return sb.toString();
  }  // toString

  public static void main(String args[])
        throws NumberFormatException, Proof_exception,
	IOException, Cons_exception {
    if (args.length != 1) {
      System.out.println("need filename");
      System.exit(-1);
    }
    Cons c = Cons.get_list_from_file(args[0]);
    Proof_object po = new Proof_object(c);

    System.out.println(po.toString());
  }  // main
}  // Proof_object class

//////////////////////////////////////////////////////// Proof_node

class Proof_node {

  // Note that this is an upside-down tree: one child and many parents.
  // Actually, it is a graph, because parents can be shared.

  Proof_node parents[];

  String id;    // proof object identifier
  String id2;   // another identifier (probably Otter or EQP ID)

  int type;
  final static int INPUT       = 1;
  final static int RESOLVE     = 2;
  final static int MERGE       = 3;
  final static int INSTANTIATE = 4;
  final static int PARAMOD     = 5;
  final static int FLIP        = 6;

  Vector positions[];
  Cons substitution;
  Cons clause;

  public Proof_node (Cons c, Hashtable ht) 
       throws NumberFormatException, Proof_exception {
    
    id         = c.head.toString();
    Cons just  = c.tail.head;
    clause     = c.tail.tail.head;

    if (c.length() >= 4 && c.tail.tail.tail.head.length() > 0)
      id2 = c.tail.tail.tail.head.head.toString();
    else
      id2 = null;

    if (just.head.label.equals("input")) {
      type = INPUT;
      parents = new Proof_node[0];  positions = new Vector[0];
    } 
    else if (just.head.label.equals("resolve")) {
      type = RESOLVE;
      parents = new Proof_node[2];  positions = new Vector[2];
      parents[0]   = my_get(ht, just.tail.head.label);
      positions[0] = just.tail.tail.head.to_vector();
      parents[1]   = my_get(ht, just.tail.tail.tail.head.label);
      positions[1] = just.tail.tail.tail.tail.head.to_vector();
    }
    else if (just.head.label.equals("merge")) {
      type = MERGE;
      parents = new Proof_node[1];  positions = new Vector[1];
      parents[0]   = my_get(ht, just.tail.head.label);
      positions[0] = just.tail.tail.head.to_vector();
    }
    else if (just.head.label.equals("instantiate")) {
      type = INSTANTIATE;
      parents = new Proof_node[1];  positions = new Vector[1];
      parents[0]   = my_get(ht, just.tail.head.label);
      substitution = just.tail.tail.head;
    }
    else if (just.head.label.equals("paramod")) {
      type = PARAMOD;
      parents = new Proof_node[2];  positions = new Vector[2];
      parents[0]   = my_get(ht, just.tail.head.label);
      positions[0] = just.tail.tail.head.to_vector();
      parents[1]   = my_get(ht, just.tail.tail.tail.head.label);
      positions[1] = just.tail.tail.tail.tail.head.to_vector();
    }
    else if (just.head.label.equals("flip")) {
      type = FLIP;
      parents = new Proof_node[1];  positions = new Vector[1];
      parents[0]   = my_get(ht, just.tail.head.label);
      positions[0] = just.tail.tail.head.to_vector();
    }
    else {
      throw (new Proof_exception("Proof obect error: " + c));
    }
    // For each position, decrement first element
    for (int i = 0; i < positions.length; i++) {
      if (positions[i] != null) {
	int n = ((Integer) positions[i].elementAt(0)).intValue();
	positions[i].setElementAt(new Integer(n - 1), 0);
      }
    }
  }  // Proof_node constructor

  private static Proof_node my_get(Hashtable ht, String s)
       throws Proof_exception {
    Proof_node pn = (Proof_node) ht.get(s);
    if (pn == null)
      throw (new Proof_exception("Proof obect, missing parent: " + s));
    else
      return pn;
  }  // my_get

  public String toString() {
    StringBuffer sb = new StringBuffer();

    sb.append("(" + id + " ");
    switch (type) {
    case INPUT:
      sb.append("(input) ");
      break;
    case RESOLVE:
      sb.append("(resolve " +
		parents[0].id + " " + vec_to_string(positions[0]) + " " +
		parents[1].id + " " + vec_to_string(positions[1]) + ") ");
      break;
    case MERGE:
      sb.append("(merge " + parents[0].id + " " +
		vec_to_string(positions[0]) + ") ");
      break;
    case INSTANTIATE:
      sb.append("(instantiate " + parents[0].id + " " + substitution + ") ");
      break;
    case PARAMOD:
      sb.append("(paramod " +
		parents[0].id + " " + vec_to_string(positions[0]) + " " +
		parents[1].id + " " + vec_to_string(positions[1]) + ") ");
      break;
    case FLIP:
      sb.append("(flip " + parents[0].id + " " +
		vec_to_string(positions[0]) + ") ");
      break;
    }
    sb.append(clause + ")\n");
    return sb.toString();
  }  // toString

  private static String vec_to_string(Vector v) {
    StringBuffer sb = new StringBuffer();
    sb.append("(");
    for (int i = 0; i < v.size(); i++) {
      if (i != 0)
	sb.append(" ");
      sb.append(((Integer) v.elementAt(i)).toString());
    }
    sb.append(")");
    return sb.toString();
  }  // vec_to_string

}  // Proof_node class

class Proof_exception extends Exception {
  public Proof_exception() { super(); }
  public Proof_exception(String s) { super(s); }
}  // Proof_exception
