import java.io.*;
import java.util.Vector;

public class Proof_step {

  String id;

  int type;
  final static int INPUT       = 1;
  final static int RESOLVE     = 2;
  final static int MERGE       = 3;
  final static int INSTANTIATE = 4;
  final static int PARAMOD     = 5;
  final static int FLIP        = 6;

  Cons clause;
  Cons substitution;

  String parent_1;
  String parent_2;

  Vector pos_1;
  Vector pos_2;

  public Proof_step (Cons c) throws NumberFormatException {
    
    id        = c.head.toString();
    Cons just = c.tail.head;
    clause    = c.tail.tail.head;

    if (just.head.label.equals("input")) {
      type = INPUT;
    } 
    else if (just.head.label.equals("resolve")) {
      type = RESOLVE;
      parent_1 = just.tail.head.label;
      pos_1    = just.tail.tail.head.to_vector();
      parent_2 = just.tail.tail.tail.head.label;
      pos_2    = just.tail.tail.tail.tail.head.to_vector();
    }
    else if (just.head.label.equals("merge")) {
      type = MERGE;
      parent_1 = just.tail.head.label;
      pos_1    = just.tail.tail.head.to_vector();
    }
    else if (just.head.label.equals("instantiate")) {
      type = INSTANTIATE;
      parent_1     = just.tail.head.label;
      substitution = just.tail.tail.head;
    }
    else if (just.head.label.equals("paramod")) {
      type = PARAMOD;
      parent_1 = just.tail.head.label;
      pos_1    = just.tail.tail.head.to_vector();
      parent_2 = just.tail.tail.tail.head.label;
      pos_2    = just.tail.tail.tail.tail.head.to_vector();
    }
    else if (just.head.label.equals("flip")) {
      type = FLIP;
      parent_1 = just.tail.head.label;
      pos_1    = just.tail.tail.head.to_vector();
    }
    else {
      System.err.println("Proof step constructor, bad cons: " + c.toString());
      System.exit(-1);
    }
  }  // Proof_step constructor

  public String toString() {
    StringBuffer sb = new StringBuffer();

    sb.append("(" + id + " ");
    switch (type) {
    case INPUT:
      sb.append("(input) ");
      break;
    case RESOLVE:
      sb.append("(resolve " + parent_1 + " " + pos_1 + " " +
		parent_2 + " " + pos_2 + ") ");
      break;
    case MERGE:
      sb.append("(merge " + parent_1 + " " + pos_1 + ") ");
      break;
    case INSTANTIATE:
      sb.append("(instantiate " + parent_1 + " " + substitution + ") ");
      break;
    case PARAMOD:
      sb.append("(paramod " + parent_1 + " " + pos_1 + " " +
		parent_2 + " " + pos_2 + ") ");
      break;
    case FLIP:
      sb.append("(flip " + parent_1 + " " + pos_1 + ") ");
      break;
    }
    sb.append(clause + ")");
    return sb.toString();
  }  // toString

  public static void main(String args[]) throws NumberFormatException {
    if (args.length != 1) {
      System.out.println("need filename");
      System.exit(-1);
    }
    Cons c = Cons.get_list_from_file(args[0]);
    for (Cons c1 = c; !c1.end(); c1 = c1.tail) {
      System.out.println("cons: " + c1.head);
      Proof_step ps = new Proof_step(c1.head);
      System.out.println("step: " + ps);
    }
  }  // main

}  // Proof_step class
