import java.io.*;
import java.awt.*;
import java.util.Vector;

public class Prooftran extends Frame {

  Proof_object proof;
  Draw_panel draw_panel;
  Label message;
  MenuBar menu_bar;
  Main main;

  final static int VERTICAL = 1;
  final static int HORIZONTAL = 2;
  int orientation = VERTICAL;

  final static int CLAUSE = 1;
  final static int POLYGON = 2;
  int highlight = POLYGON;

  int clause_display = Arrangement.MAIN_CLAUSES;

  Font small_font = new Font("Helvetica", Font.PLAIN, 10);
  Font medium_font = new Font("Helvetica", Font.PLAIN, 16);
  Font large_font = new Font("Helvetica", Font.PLAIN, 24);
  Font current_font = medium_font;

  // The following menu items sould be disabled for OTTER_1 proofs.

  MenuItem mi1, mi2;

  Prooftran(Main main, boolean is_applet, String filename, Cons proof_names) {
	 
    super("Proof Viewer");
    this.main = main;

    menu_bar = new MenuBar();
    Menu m;
    m = new Menu("File");
    m.add(new MenuItem("Refresh display"));
    if (is_applet) {
      for (Cons c = proof_names; !c.end(); c = c.tail)
	m.add(new MenuItem("Proof " + c.head.label));
    }
    else {
      m.add(new MenuItem("Load proof from file"));
    }
    // m.add(new MenuItem("Debug scroll"));
    m.add(new MenuItem(is_applet ? "Hide" : "Quit"));
    menu_bar.add(m);

    m = new Menu("Whole proof");
    m.add(new MenuItem("Show all clauses"));
    m.add(mi1 = new MenuItem("Show Otter clauses only"));
    m.add(new MenuItem("Show IDs only"));
    m.add(new MenuItem("Rearrange display"));
    menu_bar.add(m);

    m = new Menu("Selection");
    m.add(new MenuItem("Show all clauses in subproof"));
    m.add(mi2 = new MenuItem("Show only Otter clauses in subproof"));
    m.add(new MenuItem("Show only IDs in subproof"));
    m.add(new MenuItem("-"));
    m.add(new MenuItem("Show clause at root of subproof"));
    m.add(new MenuItem("Show ID at root of subproof"));
    m.add(new MenuItem("-"));
    m.add(new MenuItem("Abbreviate subproof"));
    m.add(new MenuItem("Unabbreviate one level of subproof"));
    m.add(new MenuItem("Unabbreviate whole subproof"));
    menu_bar.add(m);

    m = new Menu("Font size");
    m.add(new MenuItem("Small"));
    m.add(new MenuItem("Medium"));
    m.add(new MenuItem("Large"));
    menu_bar.add(m);

    m = new Menu("Help");
    m.add(new MenuItem("Input"));
    m.add(new MenuItem("Selection"));
    m.add(new MenuItem("Clause display"));
    m.add(new MenuItem("Abbreviation"));
    menu_bar.add(m);
    menu_bar.setHelpMenu(m);

    setMenuBar(menu_bar);

    setLayout(new BorderLayout());

    draw_panel    = new Draw_panel(this);
    message       = new Label();
    message_reset();

    Panel button_panel  = new Panel();
    button_panel.setLayout(new FlowLayout(FlowLayout.LEFT));

    Choice choice;

    button_panel.add(new Label("Orientation:", Label.RIGHT));
    choice = new Choice();
    choice.addItem("Vertical");
    choice.addItem("Horizontal");
    choice.select(orientation == HORIZONTAL ? "Horizontal" : "Vertical");
    button_panel.add(choice);

    button_panel.add(new Label("Highlight:", Label.RIGHT));
    choice = new Choice();
    choice.addItem("Clause");
    choice.addItem("Subproof");
    choice.select(highlight == CLAUSE ? "Clause" : "Subproof");
    button_panel.add(choice);

    add("Center", draw_panel);
    add("North", button_panel);
    add("South", message);


    Cons c = null;

    if (is_applet || filename == null)
      c = null;
    else
      c = get_proof_object_from_file(filename);

    if (c != null) {
      proof = build_proof_object(c);
      arrange();
    }
  }  // Prooftran constructor

  public void strong_message(String mess) {
    message.setFont(new Font("Helvetica", Font.ITALIC, 16));
    message.setForeground(Color.red);
    message.setAlignment(Label.CENTER);
    message.setText(mess);
  }

  public void regular_message(String mess) {
    message.setFont(new Font("Helvetica", Font.PLAIN, 16));
    message.setForeground(Color.black);
    message.setAlignment(Label.LEFT);
    message.setText(mess);
  }

  public void message_reset() {
    regular_message("Messages");
  }

  public boolean handleEvent(Event e) {

    if (e.id == Event.ACTION_EVENT) {

      if (e.target instanceof Choice) {

	// System.out.println("Choice event: " + e.toString());

	String label = (String) e.arg;

	if (label.equals("Vertical")) {
	  if (orientation != VERTICAL) {
	    orientation = VERTICAL;
	    re_arrange();
	  }
	}
	else if (label.equals("Horizontal")) {
	  if (orientation != HORIZONTAL) {
	    orientation = HORIZONTAL;
	    re_arrange();
	  }
	}
	else if (label.equals("Clause")) {
	  highlight = CLAUSE;
	  draw_panel.show(false);
	  draw_panel.show(true);
	}
	else if (label.equals("Subproof")) {
	  highlight = POLYGON;
	  draw_panel.show(false);
	  draw_panel.show(true);
	}
	return true;
      }  // Choice

      else if (e.target instanceof MenuItem) {
	String label = (String) e.arg;

	// File menu

	if (label.equals("Refresh display")) {
	  draw_panel.show(false);
	  draw_panel.show(true);
	} 
	else if (label.equals("Quit")) {
	  System.exit(0);
	} 
	else if (label.equals("Debug scroll")) {
	  regular_message(draw_panel.cv_width + " " + draw_panel.cv_height + " "
		  + draw_panel.virt_width + " " + draw_panel.virt_height + " "
		  + draw_panel.xoff + " " + draw_panel.yoff);
	} 
	else if (label.equals("Hide")) {
	  hide();
	} 
	else if (label.equals("Load proof from file")) {
	  get_file_and_load();
	}
	else if (label.length() > 7 && label.substring(0,6).equals("Proof ")) {
	  String pname = label.substring(6);

	  String s = main.getParameter(pname);

	  if (s == null) {
	    strong_message("Proof " + pname + " not found.");
	    return true;
	  }
	  Cons c = null;
	  try { c = Cons.build_cons_from_string(s); }
	  catch (Cons_exception exc) {
	    strong_message("Error parsing proof " + pname + ".");
	    return true;
	  }

	  proof = build_proof_object(c);
	  arrange();
	}

	// Font menu

	else if (label.equals("Small")) {
	  font_reset(small_font);
	} 
	else if (label.equals("Medium")) {
	  font_reset(medium_font);
	}
	else if (label.equals("Large")) {
	  font_reset(large_font);
	}

	// Help menu

	else if (label.equals("Input")) {
	  Helper h = new Helper("Proof Viewer Help",Help_string.s("Input"));
	}
	else if (label.equals("Selection")) {
	  Helper h = new Helper("Proof Viewer Help",Help_string.s("Selection"));
	}
	else if (label.equals("Clause display")) {
	  Helper h = new Helper("Proof Viewer Help",Help_string.s("Clause display"));
	}
	else if (label.equals("Abbreviation")) {
	  Helper h = new Helper("Proof Viewer Help",Help_string.s("Abbreviation"));
	}

	// Whole proof menu

	else if (label.equals("Show all clauses")) {
	  if (draw_panel.full_arr != null) {
	    draw_panel.full_arr.reset_clause_display(Arrangement.ALL_CLAUSES);
	    re_arrange();
	  }
	}
	else if (label.equals("Show Otter clauses only")) {
	  if (draw_panel.full_arr != null) {
	    draw_panel.full_arr.reset_clause_display(Arrangement.MAIN_CLAUSES);
	    re_arrange();
	  }
	}
	else if (label.equals("Show IDs only")) {
	  if (draw_panel.full_arr != null) {
	    draw_panel.full_arr.reset_clause_display(Arrangement.ALL_IDS);
	    re_arrange();
	  }
	}
	else if (label.equals("Rearrange display")) {
	  if (draw_panel.full_arr != null) {
	    re_arrange();
	  }
	} 

	// Selection menu

	else if (label.equals("Show clause at root of subproof")) {
	  if (draw_panel.selected_arr != null) {
	    draw_panel.selected_arr.full_clause = true;
	    if (orientation == HORIZONTAL)
	      draw_panel.full_arr.transpose();
	    re_arrange_branch(draw_panel.selected_arr);
	  }
	}
	else if (label.equals("Show ID at root of subproof")) {
	  if (draw_panel.selected_arr != null) {
	    draw_panel.selected_arr.full_clause = false;
	    if (orientation == HORIZONTAL)
	      draw_panel.full_arr.transpose();
	    re_arrange_branch(draw_panel.selected_arr);
	  }
	}
	else if (label.equals("Show only Otter clauses in subproof")) {
	  if (draw_panel.selected_arr != null) {
	    draw_panel.selected_arr.reset_clause_display(Arrangement.MAIN_CLAUSES);
	    if (orientation == HORIZONTAL)
	      draw_panel.full_arr.transpose();
	    re_arrange_branch(draw_panel.selected_arr);
	  }
	}
	else if (label.equals("Show all clauses in subproof")) {
	  if (draw_panel.selected_arr != null) {
	    draw_panel.selected_arr.reset_clause_display(Arrangement.ALL_CLAUSES);
	    if (orientation == HORIZONTAL)
	      draw_panel.full_arr.transpose();
	    re_arrange_branch(draw_panel.selected_arr);
	  }
	}
	else if (label.equals("Show only IDs in subproof")) {
	  if (draw_panel.selected_arr != null) {
	    draw_panel.selected_arr.reset_clause_display(Arrangement.ALL_IDS);
	    if (orientation == HORIZONTAL)
	      draw_panel.full_arr.transpose();
	    re_arrange_branch(draw_panel.selected_arr);
	  }
	}
	else if (label.equals("Abbreviate subproof")) {
	  if (draw_panel.selected_arr != null) {
	    draw_panel.selected_arr.abbreviate(getFontMetrics(draw_panel.getFont()), orientation == HORIZONTAL);
	    draw_panel.my_repaint();
	  }
	}
	else if (label.equals("Unabbreviate whole subproof")) {
	  if (draw_panel.selected_arr != null) {
	    if (orientation == HORIZONTAL)
	      draw_panel.full_arr.transpose();
	    draw_panel.selected_arr.unabbreviate(1000);
	    draw_panel.selected_arr.reset_clause_display(clause_display);
	    re_arrange_branch(draw_panel.selected_arr);
	  }
	}
	else if (label.equals("Unabbreviate one level of subproof")) {
	  if (draw_panel.selected_arr != null) {
	    if (orientation == HORIZONTAL)
	      draw_panel.full_arr.transpose();
	    draw_panel.selected_arr.unabbreviate(1);
	    draw_panel.selected_arr.reset_clause_display(clause_display);
	    re_arrange_branch(draw_panel.selected_arr);
	  }
	}
	return true;
      }  //MenuItem

      else {
	return false;  // unhandled ACTION_EVENT
      }
    }  // end of ACTION_EVENT cases

    else if (e.id == Event.WINDOW_DESTROY) {
      System.exit(-1);
      return true;
    }
    else
      return false;
  }  // handleEvent

  private void font_reset(Font new_font) {
    current_font = new_font;
    draw_panel.setFont(current_font);
    if (draw_panel.full_arr != null) {
      re_arrange();
    }
  }  // font_reset

  public void arrange() {
    if (proof == null) {
      draw_panel.full_arr = null;
      draw_panel.my_repaint();  // to get "no proof" display.
    }
    else {
      if (proof.type == Proof_object.OTTER_1) {
	mi1.disable();
	mi2.disable();
      }
      else {
	mi1.enable();
	mi2.enable();
      }
      draw_panel.setFont(current_font);
      draw_panel.selected_arr = null;

      strong_message("Arranging proof ...");  // doesn't work
      setCursor(WAIT_CURSOR);
      draw_panel.my_repaint();      // to get cursor to change
      try {Thread.currentThread().sleep(10);}  // delay to get cursor to change
      catch (InterruptedException e) { }

      draw_panel.full_arr = new Arrangement(proof.proof, new Vector(), 1000);
      draw_panel.full_arr.reset_clause_display(clause_display);
      
      re_arrange();
      if (proof.type == Proof_object.OTTER_1) {
	strong_message("This is an old-style proof object.");
      }
    }
  }  // arrange

  public void re_arrange() {
    if (draw_panel.full_arr != null) {

      boolean transpose = (orientation == HORIZONTAL);
      FontMetrics fm = getFontMetrics(draw_panel.getFont());

      strong_message("Arranging proof ...");  // can't make this appear
      setCursor(WAIT_CURSOR);
      draw_panel.my_repaint();      // to get cursor to change
      try {Thread.currentThread().sleep(10);}  // delay to get cursor to change
      catch (InterruptedException e) { }

      draw_panel.full_arr.relative_arrangement(fm, transpose);

      if (transpose)	
	draw_panel.full_arr.transpose();
      draw_panel.full_arr.absolute(draw_panel.pad, draw_panel.pad);
      draw_panel.my_repaint();
      setCursor(DEFAULT_CURSOR);
      message_reset();
    }
  }  // re_arrange

  public void re_arrange_branch(Arrangement a) {
    // strong_message("Arranging ...");
    boolean transpose = (orientation == HORIZONTAL);
    FontMetrics fm = getFontMetrics(draw_panel.getFont());
    a.relative_arrangement(fm, transpose);
    for (Arrangement a1 = a.parent; a1 != null; a1 = a1.parent)
      a1.relative_arrange_this_node(fm, transpose);
    if (transpose)	
      draw_panel.full_arr.transpose();
    draw_panel.full_arr.absolute(draw_panel.pad, draw_panel.pad);
    draw_panel.my_repaint();
    message_reset();
  }  // re_arrange_branch

  public Proof_object build_proof_object(Cons c) {
    Proof_object proof = null;
    try { proof = new Proof_object(c); }
    catch (Proof_exception exc) {
      strong_message("Error building proof object.");
      return null;
    }
    catch (NumberFormatException exc) {
      strong_message("Error building proof object (NumberFormatException).");
      return null;
    }
    return proof;
  }  // build_proof_object

  public Cons get_proof_object_from_file(String filename) {
    Cons c = null;
    try { c = Cons.get_list_from_file(filename); }
    catch (IOException exc) {
      strong_message(exc.getMessage());
      return null;
    }
    catch (Cons_exception exc) {
      strong_message("Error parsing proof object.");
      return null;
    }
    return c;
  }  // get_proof_object_from_file

  public void get_file_and_load() {
    FileDialog fd = new FileDialog(this, null, FileDialog.LOAD);
    // fd.setFilenameFilter(this);
    fd.show();
    String dir = fd.getDirectory();
    String f = fd.getFile();

    if (f != null) {
      Cons c = get_proof_object_from_file(dir + f);
      if (c != null) {
	proof = build_proof_object(c);
	arrange();
      }
    }
  }  // get_file_and_load

}  // class Prooftran

