import java.awt.*;

class Scrollable_canvas extends Panel {

  private Scrollbar hbar, vbar;

  public int cv_width, cv_height;     // actual canvas size (drawing size)
  public int virt_width, virt_height; // virtual canvas size (>= actual size)
  public int xoff = 0, yoff = 0;      // offsets from actual to virtual coords.

  public Scrollable_canvas() {
    hbar = new Scrollbar(Scrollbar.HORIZONTAL);
    vbar = new Scrollbar(Scrollbar.VERTICAL);

    this.setLayout(new BorderLayout(0,0));

    this.add("South", hbar);
    this.add("East", vbar);

  }  // constructor

  public int a2v_x(int x) { return x + xoff; }  // actual to virtual x
  public int a2v_y(int y) { return y + yoff; }  // actual to virtual y

  public int v2a_x(int x) { return x - xoff; }  // virtual to actual x
  public int v2a_y(int y) { return y - yoff; }  // virtual to actual y

  public boolean pt_visible(int x, int y) {
    return(x >= xoff && x <= xoff+cv_width &&
	   y >= yoff && y <= yoff+cv_height);
  }  // pt_visible

  public boolean rect_invisible(int x, int y, int width, int height) {
    return((x > xoff+cv_width || x+width <= xoff) ||
	   (y > yoff+cv_height || y+height <= yoff));
  }  // rect_invisible

  public void set_scrollable(int request_x, int request_y, Graphics g) {

    // This should get called for every repaint, and it usually doesn't
    // have to do any of this stuff.  But it sometimes it does, for example,
    // after scroll events or window resizes or changes in request values.

    cv_width = size().width - vbar.size().width;
    cv_height = size().height - hbar.size().height;
    virt_width = Math.max(request_x, cv_width);
    virt_height = Math.max(request_y, cv_height);
    xoff = Math.min(xoff, virt_width-cv_width);
    yoff = Math.min(yoff, virt_height-cv_height);
    re_init_scroll(hbar, xoff, cv_width, virt_width);
    re_init_scroll(vbar, yoff, cv_height, virt_height);
    g.translate(-xoff,-yoff);
  }  // set_virtual_size

  private void re_init_scroll(Scrollbar bar, int off, int cv, int virt) {
    if (cv != virt) {
      bar.setValues(off, cv, 0, virt - cv);
      bar.setPageIncrement(cv/8);
      bar.setLineIncrement(20);
      bar.show();
    }
    else
      bar.hide();
  }  // re_init_scroll

  public synchronized void reshape(int x, int y, int width, int height) {

    // This gets called automatically when the component is resized.
    // It also gets called during initialization, after the panel is
    // sized.

    super.reshape(x, y, width, height); // official stuff
    hbar.show();  // If hidden, positions don't get updated correctly.
    vbar.show();  // repaint() will hide them if necessary.
    repaint();
  }  // reshape

  public boolean handleEvent(Event evt) {
    switch(evt.id) {
    case Event.SCROLL_LINE_UP:
    case Event.SCROLL_LINE_DOWN:
    case Event.SCROLL_PAGE_UP:
    case Event.SCROLL_PAGE_DOWN:
    case Event.SCROLL_ABSOLUTE:
      xoff = hbar.getValue();
      yoff = vbar.getValue();
      repaint();
      return true;
    default:
      return super.handleEvent(evt);
    }
  }  // eventHandler

}  // class Scrollable_canvas
